@@ -6,13 +6,39 @@ use warnings;
 use strict;
 use utf8;
 use Benchmark ':all';
+use FindBin '$Bin';
+
+# Just so I can use the latest versions
+
 use lib '/home/ben/projects/json-create/blib/lib';
 use lib '/home/ben/projects/json-create/blib/arch';
+
+# Contenders
+
 use JSON::Create 'create_json';
 use JSON::XS;
 use Cpanel::JSON::XS;
 
-use FindBin '$Bin';
+# Number of repetitions. No matter how large this is made, the results
+# always vary wildly from run to run.
+
+my $count = 1000;
+# The results seem to stabilize better if we run the inner loop a
+# number of times. Still, unfortunately, setting this very large
+# doesn't stabilize the results completely.
+my $inner = 1000;
+
+print "Versions used:\n\n";
+my @modules = qw/Cpanel::JSON::XS JSON::XS JSON::Create/;
+for my $module (@modules) {
+    my $abbrev = $module;
+    $abbrev =~ s/(\w)\w+\W*/$1/g; 
+    my $version = eval "\$${module}::VERSION";
+    print "$abbrev\t$module\t$version\n";
+}
+
+# ASCII string test
+
 my $stuff = {
     captain => 'planet',
     he => "'s",
@@ -28,21 +54,25 @@ my $stuff = {
     side => "Captain Planet!",
 };
 
-my $count = 400000;
-
 header ("hash of ASCII strings");
 
 cmpthese (
     $count,
     {
 	'JC' => sub {
-	    my $x = JSON::Create::create_json ($stuff);
+	    for (1..$inner) {
+		my $x = JSON::Create::create_json ($stuff);
+	    }
 	},
 	'JX' => sub {
-	    my $x = JSON::XS::encode_json ($stuff);
+	    for (1..$inner) {
+		my $x = JSON::XS::encode_json ($stuff);
+	}
 	},
 	'CJX' => sub {
-	    my $x = Cpanel::JSON::XS::encode_json ($stuff);
+	    for (1..$inner) {
+		my $x = Cpanel::JSON::XS::encode_json ($stuff);
+	    }
 	},
     },    
 );
@@ -87,13 +117,19 @@ cmpthese (
     $count,
     {
 	'JC' => sub {
-	    my $x = JSON::Create::create_json ($h2n);
+	    for (1..$inner) {
+		my $x = JSON::Create::create_json ($h2n);
+	    }
 	},
 	'JX' => sub {
-	    my $x = JSON::XS::encode_json ($h2n);
+	    for (1..$inner) {
+		my $x = JSON::XS::encode_json ($h2n);
+	    }
 	},
 	'CJX' => sub {
-	    my $x = Cpanel::JSON::XS::encode_json ($h2n);
+	    for (1..$inner) {
+		my $x = Cpanel::JSON::XS::encode_json ($h2n);
+	    }
 	},
     },    
 );
@@ -118,17 +154,47 @@ cmpthese (
     $count,
     {
 	'JC' => sub {
-	    my $x = JSON::Create::create_json (\%unihash);
+	    for (1..$inner) {
+		my $x = JSON::Create::create_json (\%unihash);
+	    }
 	},
 	'JX' => sub {
-	    my $x = JSON::XS::encode_json (\%unihash);
+	    for (1..$inner) {
+		my $x = JSON::XS::encode_json (\%unihash);
+	    }
 	},
 	'CJX' => sub {
-	    my $x = Cpanel::JSON::XS::encode_json (\%unihash);
+	    for (1..$inner) {
+		my $x = Cpanel::JSON::XS::encode_json (\%unihash);
+	    }
 	},
     },    
 );
 
+header ("array of floats");
+
+my $floats = [1.0e-10, 0.1, 1.1, 9e9, 3.141592653,-1.0e-20,-9e19,];
+
+cmpthese (
+    $count,
+    {
+	'JC' => sub {
+	    for (1..$inner) {
+		my $x = JSON::Create::create_json ($floats);
+	    }
+	},
+	'JX' => sub {
+	    for (1..$inner) {
+		my $x = JSON::XS::encode_json ($floats);
+	    }
+	},
+	'CJX' => sub {
+	    for (1..$inner) {
+		my $x = Cpanel::JSON::XS::encode_json ($floats);
+	    }
+	},
+    },    
+);
 exit;
 
 sub header
